@@ -5,7 +5,7 @@ https://github.com/rmvanhees/pyspex.git
 
 Python class to create SPEXone Level-1 products
 
-Copyright (c) 2019-2020 SRON - Netherlands Institute for Space Research
+Copyright (c) 2019-2021 SRON - Netherlands Institute for Space Research
    All Rights Reserved
 
 License:  BSD-3-Clause
@@ -106,20 +106,24 @@ class Lv1io:
         append : bool, default=False
            do no clobber, but add new data to existing product
         """
-        self.__epoch = datetime(1970, 1, 1, tzinfo=timezone.utc)
-
-        # initialize private class-attributes
         self.product = Path(product)
         self.fid = None
 
+        # initialize private class-attributes
+        self.__epoch = datetime(1970, 1, 1, tzinfo=timezone.utc)
+
         # initialize Level-1 product
         if not append:
             if self.processing_level == 'L1A':
                 self.fid = init_l1a(product, **kwargs)
             elif self.processing_level == 'L1B':
                 self.fid = init_l1b(product, **kwargs)
+                if 'ref_date' in kwargs and kwargs['ref_date'] is not None:
+                    self.__epoch = kwargs['ref_date']
             elif self.processing_level == 'L1C':
                 self.fid = init_l1c(product, **kwargs)
+                if 'ref_date' in kwargs and kwargs['ref_date'] is not None:
+                    self.__epoch = kwargs['ref_date']
             else:
                 raise KeyError('valid processing levels are: L1A, L1B or L1C')
         else:
@@ -132,7 +136,7 @@ class Lv1io:
 
     def __repr__(self) -> str:
         class_name = type(self).__name__
-        return '{}({!r})'.format(class_name, self.product)
+        return f'{class_name}({self.product!r})'
 
     def __iter__(self):
         for attr in sorted(self.__dict__):
@@ -231,13 +235,11 @@ class Lv1io:
             if grp_name != '.':
                 if var_name not in self.fid[grp_name].groups \
                    and var_name not in self.fid[grp_name].variables:
-                    raise KeyError(
-                        'ds_name {} not present in product'.format(ds_name))
+                    raise KeyError(f'ds_name {ds_name} not present in product')
             else:
                 if var_name not in self.fid.groups \
                    and var_name not in self.fid.variables:
-                    raise KeyError(
-                        'ds_name {} not present in product'.format(ds_name))
+                    raise KeyError(f'ds_name {ds_name} not present in product')
 
             if isinstance(value, str):
                 self.fid[ds_name].setncattr(name, np.string_(value))
@@ -263,12 +265,10 @@ class Lv1io:
         var_name = str(PurePosixPath(name).name)
         if grp_name != '.':
             if var_name not in self.fid[grp_name].variables:
-                raise KeyError(
-                    'dataset {} not present in Level-1 product'.format(name))
+                raise KeyError(f'dataset {name} not present in Level-1 product')
         else:
             if var_name not in self.fid.variables:
-                raise KeyError(
-                    'dataset {} not present in Level-1 product'.format(name))
+                raise KeyError(f'dataset {name} not present in Level-1 product')
 
         return self.fid[name][:]
 
@@ -291,12 +291,10 @@ class Lv1io:
         var_name = str(PurePosixPath(name).name)
         if grp_name != '.':
             if var_name not in self.fid[grp_name].variables:
-                raise KeyError(
-                    'dataset {} not present in Level-1 product'.format(name))
+                raise KeyError(f'dataset {name} not present in Level-1 product')
         else:
             if var_name not in self.fid.variables:
-                raise KeyError(
-                    'dataset {} not present in Level-1 product'.format(name))
+                raise KeyError(f'dataset {name} not present in Level-1 product')
 
         dims = self.fid[name].get_dims()
         if not dims:
@@ -332,9 +330,9 @@ class Lv1io:
         if bin_size is not None:
             dict_attrs['bin_size_at_nadir'] = bin_size
 
-        for key in dict_attrs:
-            if dict_attrs[key] is not None:
-                self.fid.setncattr(key, dict_attrs[key])
+        for key, value in dict_attrs.items():
+            if value is not None:
+                self.fid.setncattr(key, value)
 
 
 # - class L1Aio -------------------------
@@ -474,7 +472,7 @@ class L1Aio(Lv1io):
         ----------
         allow_empty :  bool, default=False
         """
-        warn_str = ('SPEX Level-1 format check [WARNING]:'
+        warn_str = ('SPEX Level-1A format check [WARNING]:'
                     ' size of variable "{:s}" is wrong, only {:d} elements')
 
         # check image datasets
@@ -537,7 +535,7 @@ class L1Aio(Lv1io):
 
         Returns
         -------
-        numpy.ndarray with sec_of_day
+        tuple holding reference_day and numpy.ndarray with sec_of_day
         """
         # determine midnight before start measurement
         tstamp0 = self.epoch + timedelta(seconds=int(ccsds_sec[0]))
@@ -549,7 +547,7 @@ class L1Aio(Lv1io):
         sec_of_day = ccsds_sec - (reference_day - self.epoch).total_seconds()
 
         # return seconds since midnight
-        return (reference_day, sec_of_day + ccsds_subsec / 65536)
+        return reference_day, sec_of_day + ccsds_subsec / 65536
 
     def fill_time(self, ccsds_sec, ccsds_subsec, group=None) -> None:
         """
@@ -578,21 +576,21 @@ class L1Aio(Lv1io):
             self.set_dset('/image_attributes/image_CCSDS_subsec', ccsds_subsec)
             self.set_dset('/image_attributes/image_time', sec_of_day)
             self.set_attr('units',
-                          'seconds since {}'.format(reference_day.isoformat()),
+                          f'seconds since {reference_day.isoformat()}',
                           ds_name='/image_attributes/image_time')
         elif group in ('engineering_data', '/engineering_data'):
             self.set_dset('/engineering_data/HK_tlm_time', sec_of_day)
             self.set_attr('units',
-                          'seconds since {}'.format(reference_day.isoformat()),
+                          f'seconds since {reference_day.isoformat()}',
                           ds_name='/engineering_data/HK_tlm_time')
         else:
             self.set_dset('/navigation_data/att_time', sec_of_day)
             self.set_attr('units',
-                          'seconds since {}'.format(reference_day.isoformat()),
+                          f'seconds since {reference_day.isoformat()}',
                           ds_name='/navigation_data/att_time')
             self.set_dset('/navigation_data/orb_time', sec_of_day)
             self.set_attr('units',
-                          'seconds since {}'.format(reference_day.isoformat()),
+                          f'seconds since {reference_day.isoformat()}',
                           ds_name='/navigation_data/orb_time')
 
     def fill_science(self, img_data, img_hk, img_id) -> None:
@@ -748,33 +746,32 @@ class L1Bio(Lv1io):
     """
     processing_level = 'L1B'
     dset_stored = {
-        '/SENSOR_VIEWS_BANDS/viewport_index': 0,
-        '/SENSOR_VIEWS_BANDS/view_angles': 0,
-        '/SENSOR_VIEWS_BANDS/intensity_wavelengths': 0,
-        '/SENSOR_VIEWS_BANDS/intensity_bandpasses': 0,
-        '/SENSOR_VIEWS_BANDS/polarization_wavelengths': 0,
-        '/SENSOR_VIEWS_BANDS/polarization_bandpasses': 0,
-        '/SENSOR_VIEWS_BANDS/intensity_f0': 0,
-        '/SENSOR_VIEWS_BANDS/polarization_f0': 0,
         '/BIN_ATTRIBUTES/image_time': 0,
+        '/GEOLOCATION_DATA/altitude': 0,
         '/GEOLOCATION_DATA/latitude': 0,
         '/GEOLOCATION_DATA/longitude': 0,
-        '/GEOLOCATION_DATA/altitude': 0,
-        '/GEOLOCATION_DATA/altitude_variability': 0,
         '/GEOLOCATION_DATA/sensor_azimuth': 0,
         '/GEOLOCATION_DATA/sensor_zenith': 0,
         '/GEOLOCATION_DATA/solar_azimuth': 0,
         '/GEOLOCATION_DATA/solar_zenith': 0,
         '/OBSERVATION_DATA/I': 0,
         '/OBSERVATION_DATA/I_noise': 0,
-        '/OBSERVATION_DATA/q': 0,
-        '/OBSERVATION_DATA/q_noise': 0,
-        '/OBSERVATION_DATA/u': 0,
-        '/OBSERVATION_DATA/u_noise': 0,
         '/OBSERVATION_DATA/AoLP': 0,
         '/OBSERVATION_DATA/AoLP_noise': 0,
         '/OBSERVATION_DATA/DoLP': 0,
-        '/OBSERVATION_DATA/DoLP_noise': 0
+        '/OBSERVATION_DATA/DoLP_noise': 0,
+        '/OBSERVATION_DATA/Q_over_I': 0,
+        '/OBSERVATION_DATA/Q_over_I_noise': 0,
+        '/OBSERVATION_DATA/U_over_I': 0,
+        '/OBSERVATION_DATA/U_over_I_noise': 0,
+        '/SENSOR_VIEWS_BANDS/viewport_index': 0,
+        '/SENSOR_VIEWS_BANDS/intensity_wavelengths': 0,
+        '/SENSOR_VIEWS_BANDS/intensity_bandpasses': 0,
+        '/SENSOR_VIEWS_BANDS/intensity_F0': 0,
+        '/SENSOR_VIEWS_BANDS/polarization_wavelengths': 0,
+        '/SENSOR_VIEWS_BANDS/polarization_bandpasses': 0,
+        '/SENSOR_VIEWS_BANDS/polarization_F0': 0,
+        '/SENSOR_VIEWS_BANDS/view_angles': 0
     }
 
     def close(self):
@@ -815,7 +812,7 @@ class L1Bio(Lv1io):
         """
         Check variables with the same first dimension have equal sizes
         """
-        warn_str = ('SPEX Level-1 format check [WARNING]:'
+        warn_str = ('SPEX Level-1B format check [WARNING]:'
                     ' size of variable "{:s}" is wrong, only {:d} elements')
 
         # check datasets in group /SENSOR_VIEWS_BANDS
@@ -903,13 +900,6 @@ class L1Cio(Lv1io):
     """
     processing_level = 'L1C'
     dset_stored = {
-        '/SENSOR_VIEWS_BANDS/view_angles': 0,
-        '/SENSOR_VIEWS_BANDS/intensity_wavelengths': 0,
-        '/SENSOR_VIEWS_BANDS/intensity_bandpasses': 0,
-        '/SENSOR_VIEWS_BANDS/polarization_wavelengths': 0,
-        '/SENSOR_VIEWS_BANDS/polarization_bandpasses': 0,
-        '/SENSOR_VIEWS_BANDS/intensity_f0': 0,
-        '/SENSOR_VIEWS_BANDS/polarization_f0': 0,
         '/BIN_ATTRIBUTES/nadir_view_time': 0,
         '/BIN_ATTRIBUTES/view_time_offsets': 0,
         '/GEOLOCATION_DATA/latitude': 0,
@@ -921,20 +911,29 @@ class L1Cio(Lv1io):
         '/GEOLOCATION_DATA/solar_azimuth': 0,
         '/GEOLOCATION_DATA/solar_zenith': 0,
         '/OBSERVATION_DATA/obs_per_view': 0,
+        '/OBSERVATION_DATA/AoLP': 0,
+        '/OBSERVATION_DATA/AoLP_noise': 0,
+        '/OBSERVATION_DATA/DoLP': 0,
+        '/OBSERVATION_DATA/DoLP_noise': 0,
         '/OBSERVATION_DATA/I': 0,
         '/OBSERVATION_DATA/I_noise': 0,
         '/OBSERVATION_DATA/I_polsample': 0,
         '/OBSERVATION_DATA/I_polsample_noise': 0,
-        '/OBSERVATION_DATA/Q': 0,
-        '/OBSERVATION_DATA/Q_noise': 0,
-        '/OBSERVATION_DATA/U': 0,
-        '/OBSERVATION_DATA/U_noise': 0,
-        '/OBSERVATION_DATA/q': 0,
-        '/OBSERVATION_DATA/q_noise': 0,
-        '/OBSERVATION_DATA/u': 0,
-        '/OBSERVATION_DATA/u_noise': 0,
-        '/OBSERVATION_DATA/DoLP': 0,
-        '/OBSERVATION_DATA/DoLP_noise': 0
+        '/OBSERVATION_DATA/QC': 0,
+        '/OBSERVATION_DATA/QC_bitwise': 0,
+        '/OBSERVATION_DATA/QC_polsample': 0,
+        '/OBSERVATION_DATA/QC_polsample_bitwise': 0,
+        '/OBSERVATION_DATA/Q_over_I': 0,
+        '/OBSERVATION_DATA/Q_over_I_noise': 0,
+        '/OBSERVATION_DATA/U_over_I': 0,
+        '/OBSERVATION_DATA/U_over_I_noise': 0,
+        '/SENSOR_VIEWS_BANDS/intensity_bandpasses': 0,
+        '/SENSOR_VIEWS_BANDS/intensity_wavelengths': 0,
+        '/SENSOR_VIEWS_BANDS/intensity_F0': 0,
+        '/SENSOR_VIEWS_BANDS/polarization_bandpasses': 0,
+        '/SENSOR_VIEWS_BANDS/polarization_wavelengths': 0,
+        '/SENSOR_VIEWS_BANDS/polarization_F0': 0,
+        '/SENSOR_VIEWS_BANDS/view_angles': 0
     }
 
     def close(self):
@@ -975,7 +974,7 @@ class L1Cio(Lv1io):
         """
         Check variables with the same first dimension have equal sizes
         """
-        warn_str = ('SPEX Level-1 format check [WARNING]:'
+        warn_str = ('SPEX Level-1C format check [WARNING]:'
                     ' size of variable "{:s}" is wrong, only {:d} elements')
 
         # check datasets in group /SENSOR_VIEWS_BANDS
